/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.schemes;

import jalview.analysis.Conservation;
import jalview.datamodel.AnnotatedCollectionI;
import jalview.datamodel.ProfilesI;
import jalview.datamodel.SequenceCollectionI;
import jalview.datamodel.SequenceI;

import java.awt.Color;
import java.util.Map;

public interface ColourSchemeI
{
  /**
   * Returns the colour for the given character. For use when the colour depends
   * only on the symbol.
   * 
   * @param c
   * @return
   */
  Color findColour(char c);

  /**
   * Returns the possibly context dependent colour for the given symbol at the
   * aligned position in the given sequence. For example, the colour may depend
   * on the symbol's relationship to the consensus residue for the column.
   * 
   * @param symbol
   * @param position
   * @param seq
   * @return
   */
  Color findColour(char symbol, int position, SequenceI seq);

  /**
   * Assigns the given consensus profile for the colourscheme
   */
  void setConsensus(ProfilesI hconsensus);

  /**
   * Assigns the given conservation to the colourscheme
   * 
   * @param c
   */
  void setConservation(Conservation c);

  /**
   * Enable or disable conservation shading for this colourscheme
   * 
   * @param conservationApplied
   */
  void setConservationApplied(boolean conservationApplied);

  /**
   * Answers true if conservation shading is enabled for this colourscheme
   * 
   * @return
   */
  boolean conservationApplied();

  /**
   * Sets the scale factor for bleaching of colour in unconserved regions
   * 
   * @param i
   */
  void setConservationInc(int i);

  /**
   * Returns the scale factor for bleaching colour in unconserved regions
   * 
   * @return
   */
  int getConservationInc();

  /**
   * Returns the percentage identity threshold for applying colourscheme
   * 
   * @return
   */
  int getThreshold();

  /**
   * Sets the percentage identity threshold and type of %age identity
   * calculation for shading
   * 
   * @param pct
   *          0..100 percentage identity for applying this colourscheme
   * @param ignoreGaps
   *          when true, calculate PID without including gapped positions
   */
  void setThreshold(int pct, boolean ignoreGaps);

  /**
   * Recalculate dependent data using the given sequence collection, taking
   * account of hidden rows
   * 
   * @param alignment
   * @param hiddenReps
   */
  void alignmentChanged(AnnotatedCollectionI alignment,
          Map<SequenceI, SequenceCollectionI> hiddenReps);

  /**
   * Creates and returns a new instance of the colourscheme configured to colour
   * the given collection. Note that even simple colour schemes should return a
   * new instance for each call to this method, as different instances may have
   * differing shading by consensus or percentage identity applied.
   * 
   * @param sg
   * @param hiddenRepSequences
   * @return copy of current scheme with any inherited settings transferred
   */
  ColourSchemeI getInstance(AnnotatedCollectionI sg,
          Map<SequenceI, SequenceCollectionI> hiddenRepSequences);

  /**
   * Answers true if the colour scheme is suitable for the given data, else
   * false. For example, some colour schemes are specific to either peptide or
   * nucleotide, or only apply if certain kinds of annotation are present.
   * 
   * @param ac
   * @return
   */
  // TODO can make this method static in Java 8
  boolean isApplicableTo(AnnotatedCollectionI ac);

  /**
   * Answers the 'official' name of the colour scheme (as used, for example, as
   * a Jalview startup parameter)
   * 
   * @return
   */
  String getSchemeName();

  /**
   * Answers true if the colour scheme depends only on the sequence symbol, and
   * not on other information such as alignment consensus or annotation. (Note
   * that simple colour schemes may have a fading by percentage identity or
   * conservation overlaid.) Simple colour schemes can be propagated to
   * structure viewers.
   * 
   * @return
   */
  boolean isSimple();
}
